/*
 * Copyright (C) 2003 - 2007 NetXen, Inc.
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 * 
 * The full GNU General Public License is included in this distribution
 * in the file called LICENSE.
 * 
 * Contact Information:
 *    licensing@netxen.com
 * NetXen, Inc.
 * 3965 Freedom Circle, Fourth floor,
 * Santa Clara, CA 95054
 */
#ifndef _UNM_NIC_
#define _UNM_NIC_

#include <linux/module.h>
#include <linux/version.h>
#ifndef __VMKERNEL_MODULE__
#include <linux/autoconf.h>
#include <linux/interrupt.h>
#endif

#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#define MODVERSIONS
#endif

#if defined(MODVERSIONS) && !defined(__GENKSYMS__)
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#ifndef __VMKERNEL_MODULE__
#include <linux/modversions.h>
#endif /* __VMKERNEL_MODULE__ */
#endif
#endif

#include <linux/skbuff.h>
#ifdef NETIF_F_HW_VLAN_TX
#include <linux/if_vlan.h>
#endif
#ifndef MODULE
#include <asm/i387.h>
#endif
#include <linux/if_arp.h>
#include "unm_nic_hw.h"
#include "nic_cmn.h"
#include "unm_inc.h"      /* For MAX_RCV_CTX */
#include "unm_pstats.h"
#include "unm_brdcfg.h"
#include "queue.h"
#include "nx_nic_linux_tnic_api.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,28)
#define PCI_DEVICE(vend,dev) \
         .vendor = (vend), .device = (dev), \
         .subvendor = PCI_ANY_ID, .subdevice = PCI_ANY_ID
#define netdev_priv(dev) (dev->priv)
#endif

#define BAR_0            0
#define PCI_DMA_64BIT    0xffffffffffffffffULL
#define PCI_DMA_32BIT    0x00000000ffffffffULL

#define NETDEV_STATUS 0x1
#ifndef __VMKERNEL_MODULE__
#define BOUNCE_BUFFERS
#endif /* !__VMKERNEL_MODULE__ */

#define ADDR_IN_WINDOW0(off)    \
       ((off >= UNM_CRB_PCIX_HOST) && (off < UNM_CRB_PCIX_HOST2)) ? 1 : 0
#define ADDR_IN_WINDOW1(off)   \
       ((off > UNM_CRB_PCIX_HOST2) && (off < UNM_CRB_MAX)) ? 1 : 0

/*Do not use CRB_NORMALIZE if this is true*/
#define ADDR_IN_BOTH_WINDOWS(off)\
       ((off >= UNM_CRB_PCIX_HOST) && (off < UNM_CRB_DDR_NET)) ? 1 : 0

#if defined(CONFIG_X86_64) || defined(CONFIG_64BIT)
typedef unsigned long uptr_t;
#else
typedef unsigned uptr_t;
#endif

#define FIRST_PAGE_GROUP_START	0
#define FIRST_PAGE_GROUP_END	0x100000

#define SECOND_PAGE_GROUP_START	0x6000000
#define SECOND_PAGE_GROUP_END	0x68BC000

#define THIRD_PAGE_GROUP_START	0x70E4000
#define THIRD_PAGE_GROUP_END	0x8000000

#define FIRST_PAGE_GROUP_SIZE	FIRST_PAGE_GROUP_END - FIRST_PAGE_GROUP_START
#define SECOND_PAGE_GROUP_SIZE	SECOND_PAGE_GROUP_END - SECOND_PAGE_GROUP_START
#define THIRD_PAGE_GROUP_SIZE	THIRD_PAGE_GROUP_END - THIRD_PAGE_GROUP_START

/* normalize a 64MB crb address to 32MB PCI window
 * To use CRB_NORMALIZE, window _must_ be set to 1
 */
#define CRB_NORMAL(reg)	\
	(reg) - UNM_CRB_PCIX_HOST2 + UNM_CRB_PCIX_HOST
#define CRB_NORMALIZE(adapter, reg) \
	(void *)(uptr_t)(pci_base_offset(adapter, CRB_NORMAL(reg)))

#define DB_NORMALIZE(adapter, off) \
        (void *)(uptr_t)(adapter->ahw.db_base + (off))


/* Phantom is multi-function.  For our purposes,  use function 0 */
#define PHAN_FUNC_NUM         0

#define UNM_WRITE_LOCK(lock) \
        do { \
                write_lock((lock)); \
        } while (0)

#define UNM_WRITE_UNLOCK(lock) \
        do { \
                write_unlock((lock)); \
        } while (0)

#define UNM_READ_LOCK(lock) \
        do { \
                read_lock((lock)); \
        } while (0)

#define UNM_READ_UNLOCK(lock) \
        do { \
                read_unlock((lock)); \
        } while (0)

#define UNM_WRITE_LOCK_IRQS(lock, flags)\
        do { \
                write_lock_irqsave((lock), flags); \
        } while (0)

#define UNM_WRITE_UNLOCK_IRQR(lock, flags)\
        do { \
                write_unlock_irqrestore((lock), flags); \
        } while (0)

#define PFX "unm: "

/* FIXME vijo */
extern int unm_nic_debug;
extern char unm_nic_driver_name[];
#define unm_nic_debug_on  do { unm_nic_debug = 1; } while(0)
#define unm_nic_debug_off do { unm_nic_debug = 0; } while(0)

/* FIXME, vijo: port[0] is not correct */
/* Note: Make sure to not call this before adapter->port is valid */
#if defined(NDEBUG)
#define DPRINTK(nlevel, klevel, fmt, args...)  do { \
    } while (0)
#else /* NDEBUG */
#define DPRINTK(nlevel, klevel, fmt, args...)  do { \
    /* (void)((NETIF_MSG_##nlevel & adapter->msg_enable) && */\
    if (unm_nic_debug) \
    printk(KERN_##klevel PFX "%s: %s: " fmt, __FUNCTION__,\
           (adapter != NULL && adapter->netdev != NULL) ? \
            adapter->netdev->name : NULL, \
              ## args); } while(0)
#endif

#define UNM_MAX_INTR        10

#define UNM_RXBUFFER        2048

/* Number of status descriptors to handle per interrupt */
#define MAX_STATUS_HANDLE  (128)

/* Number of status descriptors to process before posting buffers */
#define MAX_RX_THRESHOLD  (MAX_STATUS_HANDLE*2)

/* netif_wake_queue ?  count*/
#define UNM_TX_QUEUE_WAKE      16

/* After UNM_RX_BUFFER_WRITE have been processed, we will repopulate them */
#define UNM_RX_BUFFER_WRITE    64

/* only works for sizes that are powers of 2 */
#define UNM_ROUNDUP(i, size) ((i) = (((i) + (size) - 1) & ~((size) - 1)))

#ifndef    TAILQ_FIRST
#define    TAILQ_FIRST(head)    ((head)->tqh_first)
#define    TAILQ_EMPTY(head)    ((head)->tqh_first == NULL)
#endif

/* For the MAC Address:experimental ref. pegnet_types.h */
#define UNM_MAC_OUI          "00:0e:1e:"
#define UNM_MAC_EXPERIMENTAL ((unsigned char)0x20)

/*
 * unm_skb_frag{} is to contain mapping info for each SG list. This
 * has to be freed when DMA is complete. This is part of unm_tx_buffer{}.
 */
struct    unm_skb_frag {
        uint64_t    dma;
        uint32_t    length;
};

#ifdef BOUNCE_BUFFERS
/* Bounce buffer index */
struct bounce_index {
       /* Index of a buffer */
       unsigned buffer_index;
       /* Offset inside the buffer */
       unsigned buffer_offset;
};

#define IS_BOUNCE 0xcafebb
#endif

/*    Following defines are for the state of the buffers    */
#define    UNM_BUFFER_FREE        0
#define    UNM_BUFFER_BUSY        1

/*
 * There will be one unm_buffer per skb packet.    These will be
 * used to save the dma info for pci_unmap_page()
 */
struct unm_cmd_buffer {
        TAILQ_ENTRY(unm_cmd_buffer)    link;

        struct sk_buff         *skb;
        struct unm_skb_frag     fragArray[MAX_BUFFERS_PER_CMD+1];
        uint32_t                totalLength;
        uint32_t                mss;
        uint32_t                port:16,
                                cmd:8,
                                fragCount:8;
        unsigned long           time_stamp;

        uint32_t                state;
#ifdef BOUNCE_BUFFERS
        uint32_t                tx_bounce_buff;
        struct bounce_index     bnext;
#endif
};

/* In rx_buffer, we do not need multiple fragments as is a single buffer */
struct unm_rx_buffer {
        TAILQ_ENTRY(unm_rx_buffer)    link;

        struct sk_buff     *skb;
        uint64_t            dma;
        uint32_t            refHandle:16,
                            state:16;
        uint32_t            lro_expected_frags;
        uint32_t            lro_current_frags;
        uint32_t            lro_length;
#ifdef BOUNCE_BUFFERS
        uint32_t            rx_bounce_buff;
        struct bounce_index bnext;
        char*               bounce_ptr;
#endif
};

/* Board types */
#define  UNM_NIC_GBE     0x01
#define  UNM_NIC_XGBE    0x02

/*
 * One hardware_context{} per adapter
 * contains interrupt info as well shared hardware info.
 */
typedef struct _hardware_context {
        struct pci_dev   *pdev;
	unsigned long     pci_base0;
	unsigned long     pci_len0;
	unsigned long     pci_base1;
	unsigned long     pci_len1;
	unsigned long     pci_base2;
	unsigned long     pci_len2;
	unsigned long     first_page_group_end;
	unsigned long     first_page_group_start;
        uint16_t          vendor_id;
        uint16_t          device_id;
        uint8_t           revision_id;
        uint16_t          pci_cmd_word;
        uint16_t          board_type;
        int               pci_func;
        uint16_t          max_ports;
        unm_board_info_t  boardcfg;
        uint32_t          xg_linkup;
        uint32_t          qg_linksup;

        struct unm_adapter    *adapter;

        cmdDescType0_t  *cmdDescHead;    /* Address of cmd ring in Phantom */

        uint32_t    cmdProducer;
        uint32_t    cmdConsumer;
        uint32_t    rcvFlag;
        uint32_t    statusRxProducer;
        uint32_t    statusRxConsumer;
        uint32_t    crb_base;
        unsigned long   db_base;   /* base of mapped db memory */
        unsigned long   db_len;    /* length of mapped db memory */

        unsigned long     LEDState; //LED Test 
        unsigned long     LEDTestRet;//LED test
        unsigned long     LEDTestLast;//LED test
        dma_addr_t      cmdDesc_physAddr;
        struct pci_dev *cmdDesc_pdev;
        int             ddr_mn_window;
        int             qdr_sn_window;
} hardware_context, *phardware_context;

#define MAX_BUFFERS              2000

#define MINIMUM_ETHERNET_FRAME_SIZE  64   /* With FCS */
#define ETHERNET_FCS_SIZE             4

#if 0
struct unm_adapter_stats {
        uint64_t  ints;
        uint64_t  hostints;
        uint64_t  otherints;
        uint64_t  process_rcv;
        uint64_t  process_xmit;
        uint64_t  noxmitdone;
        uint64_t  xmitcsummed;
        uint64_t  post_called;
        uint64_t  posted;
        uint64_t  lastposted;
        uint64_t  goodskbposts;
};
#endif

struct unm_adapter_stats {
        uint64_t  rcvdbadskb;
        uint64_t  xmitcalled;
        uint64_t  xmitedframes;
        uint64_t  xmitfinished;
        uint64_t  badskblen;
        uint64_t  nocmddescriptor;
        uint64_t  polled;
        uint64_t  uphappy;
        uint64_t  updropped;
        uint64_t  uplcong;
        uint64_t  uphcong;
        uint64_t  upmcong;
        uint64_t  updunno;
        uint64_t  skbfreed;
        uint64_t  txdropped;
        uint64_t  txnullskb;
        uint64_t  csummed;
        uint64_t  no_rcv;
        uint64_t  rxbytes;
        uint64_t  txbytes;
	uint64_t  ints;
};

/*
 * Ring which will hold the skbs which can be fed to receive ring
 */
#define UNM_SKB_ARRAY_SIZE 1024

typedef struct unm_recv_skb_ring_s {
	struct sk_buff **skb_array;
	uint32_t interrupt_index;
	uint32_t tasklet_index;
        uint32_t data_size;
} unm_recv_skb_ring_t;

/* descriptor types */
#define RCV_RING_STD       RCV_DESC_NORMAL
#define RCV_RING_JUMBO      RCV_DESC_JUMBO
#define RCV_RING_LRO            RCV_DESC_LRO

/*
 * Rcv Descriptor Context. One such per Rcv Descriptor. There may
 * be one Rcv Descriptor for normal packets, one for jumbo,
 * one for LRO and may be expanded.
 */
typedef struct unm_rcv_desc_context_s {
        uint32_t              flags;
        uint32_t              producer;

        /* Num of bufs posted in phantom */
        uint32_t              rcv_pending;
        /* Num of bufs in free list */
        uint32_t              rcv_free;

        dma_addr_t              phys_addr;
        struct pci_dev        *phys_pdev;
        /* address of rx ring in Phantom */
        rcvDesc_t            *desc_head;

        uint32_t              MaxRxDescCount;
        uint32_t              dma_size;
        uint32_t              skb_size;
        /* rx buffers for receive   */
        struct unm_rx_buffer *rx_buf_arr;
        /* free list */
        TAILQ_HEAD(free_rbuf_list,unm_rx_buffer )    free_rxbuf_list;
        /* physical address of above */
        dma_addr_t            rx_buf_phys;

	int begin_alloc;
} unm_rcv_desc_ctx_t;

/*
 * Receive context. There is one such structure per instance of the
 * receive processing. Any state information that is relevant to
 * the receive, and is must be in this structure. The global data may be
 * present elsewhere.
 */
typedef struct unm_recv_context_s {
        unm_rcv_desc_ctx_t rcv_desc[NUM_RCV_DESC_RINGS];

        uint32_t      statusRxProducer;
        uint32_t      statusRxConsumer;

        dma_addr_t    rcvStatusDesc_physAddr;
        struct pci_dev *rcvStatusDesc_pdev;
        statusDesc_t *rcvStatusDescHead;
} unm_recv_context_t;

#ifdef  PEGNET_NIC
/*
 * Taken from the Windows driver. TODO: We need to use a common code between
 * linux and windows.
 * This is used when we do not have enough descriptors..
 * When the interrupt routines has freed up more than 64 descriptors, only then
 * we will send them to the hardware... or the interrupt routine has enough
 * descriptors to complete all the descriptors..
 */
#define	MAX_PENDING_DESC_BLOCK_SIZE	64

typedef struct unm_pending_cmd_desc_block_s {
	struct list_head        link;
	uint32_t                cnt;
	cmdDescType0_t          cmd[MAX_PENDING_DESC_BLOCK_SIZE];
} unm_pending_cmd_desc_block_t;

/*
 * When the current block gets full, it is moved into the cmd_list.
 */
typedef struct unm_pending_cmd_descs_s {
	uint32_t                        cnt;
	struct list_head                free_list;
	struct list_head                cmd_list;
	unm_pending_cmd_desc_block_t    *curr_block;
} unm_pending_cmd_descs_t;

#endif  /* PEGNET_NIC */


#define UNM_NIC_MSI_ENABLED 0x02


/* The structure below is used for loopback test*/
typedef struct unm_test_ctr{

	uint8_t		*tx_user_packet_data;
	uint8_t		*rx_user_packet_data;
	uint32_t	tx_user_packet_length;
	uint32_t	rx_datalen;
	uint32_t	rx_user_pos;
	uint32_t	loopback_start;
	int		capture_input;
	int		tx_stop;
}unm_test_ctr_t;

typedef struct netdev_list_s netdev_list_t;
struct netdev_list_s {
        netdev_list_t *next;
        struct net_device *netdev;
};

/* Following structure is for specific port information */
typedef struct unm_adapter_s {
        hardware_context    ahw;
	uint8_t id[32];
	uint8_t procname[8];    /* name of this device's proc entry */
	uint16_t    portnum;    /* port number        */
        uint16_t    link_speed;
        uint16_t    link_duplex;
        uint16_t    state;        /* state of the port             */
        uint16_t    link_autoneg;

        int         rx_csum;
        int         status;
        spinlock_t  stats_lock;

        struct net_device      *netdev;
        struct pci_dev         *pdev;
        struct net_device_stats net_stats;
        struct unm_adapter_stats stats;
        unsigned char    mac_addr[MAX_ADDR_LEN];
        int              mtu;

        spinlock_t          tx_lock;
	rwlock_t            adapter_lock;
        spinlock_t          lock;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
        struct work_struct  watchdog_task;
        struct work_struct  tx_timeout_task[4];
#else
        struct tq_struct    tx_timeout_task[4];
	struct tq_struct    watchdog_task;
#endif
        struct timer_list      watchdog_timer;
        struct tasklet_struct  tx_tasklet;

        uint32_t        curr_window;

        uint32_t        cmdProducer;
        uint32_t        *cmdConsumer;

        uint32_t        lastCmdConsumer;
        /* Num of bufs posted in phantom */
        uint32_t        pendingCmdCount;
        uint32_t        freeCmdCount;    /* Num of bufs in free list */
        uint32_t        MaxTxDescCount;
        uint32_t        MaxRxDescCount;
        uint32_t        MaxJumboRxDescCount;
        uint32_t        MaxLroRxDescCount;
	/* Variables for send test and loopback are in this container*/
	unm_test_ctr_t	testCtx;
        /* Num of instances active on cmd buffer ring */
        uint32_t        procCmdBufCounter;
        volatile uint32_t cmdPegConsumer;

        /*Use to keep track of xmit threads*/
        uint32_t        num_threads, total_threads;

        uint32_t        flags;
        int             driver_mismatch;
	    uint32_t	    temp;


        struct unm_cmd_buffer *cmd_buf_arr;  /* Command buffers for xmit */
        struct unm_rx_buffer  *rx_buf_arr;   /* rx buffers for receive   */

        /*
         * Receive instances. These can be either one per port,
         * or one per peg, etc.
         */
        unm_recv_context_t recv_ctx[MAX_RCV_CTX];

       struct netdev_list_s *netlist;
       int             number;
       int             is_up;

        /* context interface shared between card and host */
        RingContext     *ctxDesc;
        struct pci_dev  *ctxDesc_pdev;
        dma_addr_t      ctxDesc_physAddr;

#ifdef PEGNET_NIC
        unm_pending_cmd_descs_t pending_cmds;

#define UNM_HOST_DUMMY_DMA_SIZE         1024
        struct {
                void            *addr;
                dma_addr_t      phys_addr;
        } dummy_dma;

	struct {
		spinlock_t		lock;
		int			registered;
		nx_tnic_adapter_t	*data;
		nx_nic_lsa_registry_t	*cb;
		int			refcnt;
	} tnic;

	void			*rdma_private;
#endif /* PEGNET_NIC */
	int intr_scheme;
	int msi_mode;
        int (*unm_nic_pci_mem_write)(struct unm_adapter_s *, u64, void *, int); 
        int (*unm_nic_pci_mem_read)(struct unm_adapter_s *, u64, void *, int); 
} unm_adapter;    /* unm_adapter structure */


#define    PORT_DOWN            0
#define    PORT_UP              1
#define    PORT_INITIALIAZED    2
#define    PORT_SUSPEND         3

#define    MAX_HD_DESC_BUFFERS    4    /* number of buffers in 1st desc */

/*Max number of xmit producer threads that can run simultaneously*/
#define    MAX_XMIT_PRODUCERS   16


#define PROC_PARENT            NULL


void unm_nic_pci_change_crbwindow(unm_adapter *adapter, uint32_t wndw);

#define PCI_OFFSET_FIRST_RANGE(adapter, off)	\
	((adapter)->ahw.pci_base0 + off)
#define PCI_OFFSET_SECOND_RANGE(adapter, off)	\
	((adapter)->ahw.pci_base1 + off - SECOND_PAGE_GROUP_START)
#define PCI_OFFSET_THIRD_RANGE(adapter, off)	\
	((adapter)->ahw.pci_base2 + off - THIRD_PAGE_GROUP_START)

#define pci_base_offset(adapter, off)	\
	((((off) < ((adapter)->ahw.first_page_group_end)) &&                                            \
          ((off) >= ((adapter)->ahw.first_page_group_start))) ?			                        \
		((adapter)->ahw.pci_base0 + (off)) :							\
		((((off) < SECOND_PAGE_GROUP_END) && ((off) >= SECOND_PAGE_GROUP_START)) ?		\
			((adapter)->ahw.pci_base1 + (off) - SECOND_PAGE_GROUP_START) :			\
			((((off) < THIRD_PAGE_GROUP_END) && ((off) >= THIRD_PAGE_GROUP_START)) ?	\
				((adapter)->ahw.pci_base2 + (off) - THIRD_PAGE_GROUP_START) :		\
				0)))

#define pci_base(adapter, off)	\
	((((off) < ((adapter)->ahw.first_page_group_end)) &&                                            \
          ((off) >= ((adapter)->ahw.first_page_group_start))) ?			                        \
		((adapter)->ahw.pci_base0) :								\
		((((off) < SECOND_PAGE_GROUP_END) && ((off) >= SECOND_PAGE_GROUP_START)) ?		\
			((adapter)->ahw.pci_base1) :							\
			((((off) < THIRD_PAGE_GROUP_END) && ((off) >= THIRD_PAGE_GROUP_START)) ?	\
				((adapter)->ahw.pci_base2) :						\
				0)))

static inline void
unm_nic_reg_write (unm_adapter *adapter, u64 off, __uint32_t val)
{//Only for window 1
        void *addr;

        UNM_READ_LOCK(&adapter->adapter_lock);

        if (adapter->curr_window != 1) {
                unm_nic_pci_change_crbwindow(adapter, 1);
        }
        addr = CRB_NORMALIZE(adapter, off);
        //DPRINTK(1, INFO, "writing to base %lx offset %llx addr %p data %x\n",
        //                    pci_base(adapter, off), off, addr, val);
        UNM_NIC_PCI_WRITE_32 (val, addr);

        UNM_READ_UNLOCK(&adapter->adapter_lock);
}

static inline int
unm_nic_reg_read (unm_adapter *adapter, u64 off)
{//Only for window 1
        void *addr;
        int val;

        UNM_READ_LOCK(&adapter->adapter_lock);
        addr = CRB_NORMALIZE(adapter, off);
        //DPRINTK(1, INFO, "reading from base %lx offset %llx addr %p\n",
        //                pci_base(adapter, off), off, addr);
        val = UNM_NIC_PCI_READ_32(addr);
        UNM_NIC_PCI_WRITE_32 (val, addr);
        UNM_READ_UNLOCK(&adapter->adapter_lock);

        return val;
}

static inline void
unm_nic_write_w0(unm_adapter *adapter, uint32_t index, uint32_t value)
{//Change the window to 0, write and change back to window 1.
        unsigned long        flags;
        void                *addr;

	UNM_WRITE_LOCK_IRQS(&adapter->adapter_lock, flags);
        unm_nic_pci_change_crbwindow(adapter, 0);
        addr = (void *)(pci_base_offset(adapter, index));
        UNM_NIC_PCI_WRITE_32(value, addr);
        unm_nic_pci_change_crbwindow(adapter, 1);
	UNM_WRITE_UNLOCK_IRQR(&adapter->adapter_lock, flags);
}

static inline void
unm_nic_read_w0(unm_adapter *adapter, uint32_t index, uint32_t *value)
{//Change the window to 0, read and change back to window 1.
        unsigned long        flags;
        void                *addr;

        addr = (void *)(pci_base_offset(adapter, index));

	UNM_WRITE_LOCK_IRQS(&adapter->adapter_lock, flags);
        unm_nic_pci_change_crbwindow(adapter, 0);
        *value = UNM_NIC_PCI_READ_32(addr);
        unm_nic_pci_change_crbwindow(adapter, 1);
	UNM_WRITE_UNLOCK_IRQR(&adapter->adapter_lock, flags);
}


int unm_niu_xg_get_tx_flow_ctl(struct unm_adapter_s *adapter,
				int *val);
int unm_niu_gbe_get_tx_flow_ctl(struct unm_adapter_s *adapter,
				int *val);
int unm_niu_gbe_get_rx_flow_ctl(struct unm_adapter_s *adapter,
				int *val);
int unm_niu_xg_set_tx_flow_ctl(struct unm_adapter_s *adapter,
				int enable);
int unm_niu_gbe_set_rx_flow_ctl(struct unm_adapter_s *adapter,
				int enable);
int unm_niu_gbe_set_tx_flow_ctl(struct unm_adapter_s *adapter,
				int enable);
long unm_niu_gbe_enable_phy_interrupts(struct unm_adapter_s *);
long unm_niu_xg_enable_phy_interrupts(struct unm_adapter_s *);
long unm_niu_gbe_disable_phy_interrupts(struct unm_adapter_s *);
long unm_niu_xg_disable_phy_interrupts(struct unm_adapter_s *);
long unm_niu_gbe_clear_phy_interrupts(struct unm_adapter_s *);
long unm_niu_gbe_phy_read (struct unm_adapter_s *,
			   long reg, unm_crbword_t *readval);
long unm_niu_gbe_phy_write (struct unm_adapter_s *,
			    long reg, unm_crbword_t val);
long unm_niu_xginit(void);
int xge_loopback(struct unm_adapter_s *, int on);
int xge_link_status (struct unm_adapter_s *, long *status);
int phy_lock(struct unm_adapter_s *adapter);
void phy_unlock(struct unm_adapter_s *adapter);

/* Functions available from unm_nic_hw.c */
int unm_nic_get_board_info(struct unm_adapter_s *adapter);
void _unm_nic_write_crb (struct unm_adapter_s *adapter, uint32_t index,
                                uint32_t value);
void  unm_nic_write_crb (struct unm_adapter_s *adapter, uint32_t index,
                                uint32_t value);
void _unm_nic_read_crb  (struct unm_adapter_s *adapter, uint32_t index,
                                uint32_t *value);
void  unm_nic_read_crb  (struct unm_adapter_s *adapter, uint32_t index,
                                uint32_t *value);
int   unm_nic_reg_read (unm_adapter *adapter, u64 off);
int _unm_nic_hw_write  (struct unm_adapter_s *adapter,
                                u64 off, void *data, int len);
int  unm_nic_hw_write  (struct unm_adapter_s *adapter,
                                u64 off, void *data, int len);
void unm_nic_reg_write (unm_adapter *adapter, u64 off, __uint32_t val);
int _unm_nic_hw_read   (struct unm_adapter_s *adapter,
                                u64 off, void *data, int len);
int  unm_nic_hw_read   (struct unm_adapter_s *adapter,
                                u64 off, void *data, int len);
void _unm_nic_hw_block_read (struct unm_adapter_s *adapter,
                                u64 off, void *data, int num_words);
void  unm_nic_hw_block_read (struct unm_adapter_s *adapter,
                                u64 off, void *data, int num_words);
void _unm_nic_hw_block_write(struct unm_adapter_s *adapter,
                                u64 off, void *data, int num_words);
void unm_nic_hw_block_write(struct unm_adapter_s *adapter,
                                u64 off, void *data, int num_words);
int  unm_nic_pci_mem_write_direct(struct unm_adapter_s *adapter,
                                u64 off, void *data, int size);
int  unm_nic_pci_mem_write_agent(struct unm_adapter_s *adapter,
                                u64 off, void *data, int size);
int  unm_nic_pci_mem_read_direct(struct unm_adapter_s *adapter,
                                u64 off, void *data, int size);
int  unm_nic_pci_mem_read_agent(struct unm_adapter_s *adapter,
                                u64 off, void *data, int size);
void unm_nic_mem_block_read(struct unm_adapter_s *adapter, u64 off,
                                void *data, int num_words);
void unm_nic_mem_block_write(struct unm_adapter_s *adapter, u64 off,
                                void *data, int num_words);
int unm_nic_hw_read_wx (unm_adapter *adapter, u64 off, void *data, int len);
int unm_nic_hw_write_wx(unm_adapter *adapter, u64 off, void *data, int len);
int unm_nic_hw_read_ioctl(unm_adapter *adapter, u64 off, void *data, int len);
int unm_nic_hw_write_ioctl(unm_adapter *adapter, u64 off, void *data, int len);
int  unm_nic_macaddr_set(struct unm_adapter_s *adapter, __uint8_t *addr);
void unm_tcl_resetall(struct unm_adapter_s *adapter);
void unm_tcl_phaninit(struct unm_adapter_s *adapter);
void unm_tcl_postimage(struct unm_adapter_s *adapter);
//int update_core_clock(struct unm_port *port);
int unm_nic_set_mtu(struct unm_adapter_s *adapter, int new_mtu);
long unm_nic_phy_read(unm_adapter *adapter, long reg, __uint32_t *);
long unm_nic_phy_write(unm_adapter *adapter, long reg, __uint32_t);
long unm_nic_init_port(struct unm_adapter_s *adapter);
int unm_crb_write_adapter (unsigned long off, void *data, struct unm_adapter_s *adapter);
int unm_crb_read_adapter (unsigned long, void *, struct unm_adapter_s *);
int unm_crb_read_val_adapter (unsigned long , struct unm_adapter_s *);
int unm_crb_writelit_adapter (struct unm_adapter_s *, unsigned long, int);

/* Functions from unm_nic_init.c */
int phantom_init(struct unm_adapter_s *adapter, int first_time);
int load_firmware(struct unm_adapter_s *adapter);
int  pinit_from_rom(unm_adapter *adapter, int verbose);
int  rom_fast_read(struct unm_adapter_s *adapter, int addr, int *valp);

/* Functions from unm_nic_isr.c */
int unm_link_ok(struct unm_adapter_s *adapter);
void unm_nic_isr_other(unm_adapter *adapter);
void unm_nic_handle_phy_intr(unm_adapter *adapter);
long unm_nic_enable_phy_interrupts(unm_adapter *adapter);
long unm_nic_disable_phy_interrupts(unm_adapter *adapter);
long unm_nic_clear_phy_interrupts(unm_adapter *adapter);
void unm_indicate_link_status(unm_adapter *adapter, u32 link);
void unm_handle_port_int(unm_adapter *adapter, u32 enable);
int  tap_crb_mbist_clear(unm_adapter *adapter);
int  unm_nic_set_promisc_mode(struct unm_adapter_s *adapter);
int  unm_nic_unset_promisc_mode(struct unm_adapter_s *adapter);
void unm_nic_stop_port(struct unm_adapter_s *adapter);
int  unm_nic_get_board_num(unm_adapter *adapter);
/* Functions from xge_mdio.c */
long unm_xge_mdio_rd(struct unm_adapter_s *adapter, long devaddr, long addr);
void unm_xge_mdio_wr(struct unm_adapter_s *adapter, long devaddr, long addr, long data);

/* Functions from niu.c */

/* Set promiscuous mode for a GbE interface */
native_t unm_niu_set_promiscuous_mode(struct unm_adapter_s *,
				      unm_niu_prom_mode_t mode);
native_t unm_niu_xg_set_promiscuous_mode(struct unm_adapter_s *,
                                         unm_niu_prom_mode_t mode);

/* XG versons */
int unm_niu_xg_macaddr_get(struct unm_adapter_s *,
			   unm_ethernet_macaddr_t *addr);
int unm_niu_xg_macaddr_set(struct unm_adapter_s *,
			   unm_ethernet_macaddr_t addr);

/* Generic enable for GbE ports. Will detect the speed of the link. */
long unm_niu_gbe_init_port(long port);

/* Disable a GbE interface */
native_t unm_niu_disable_gbe_port(struct unm_adapter_s *);

native_t unm_niu_disable_xg_port(struct unm_adapter_s *);

/* get/set the MAC address for a given MAC */
int unm_niu_macaddr_get(struct unm_adapter_s *adapter,
                        unsigned char *addr);
int unm_niu_macaddr_set(struct unm_adapter_s *adapter,
                        unm_ethernet_macaddr_t addr);

extern int physical_port[];	/* physical port # from virtual port. */
#endif
