/*
 * Copyright (C) 2003 - 2007 NetXen, Inc.
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 * 
 * The full GNU General Public License is included in this distribution
 * in the file called LICENSE.
 * 
 * Contact Information:
 *    licensing@netxen.com
 * NetXen, Inc.
 * 3965 Freedom Circle, Fourth floor,
 * Santa Clara, CA 95054
 */
#ifndef __KERNEL_COMPATIBILITY_H__
#define __KERNEL_COMPATIBILITY_H__

#include <linux/version.h>
#include <net/sock.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,7)
#define	NX_KC_ROUTE_DEVICE(RT)	((RT)->u.dst.dev)
#else
#define	NX_KC_ROUTE_DEVICE(RT)	(((RT)->idev)->dev)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9)
extern kmem_cache_t *nx_sk_cachep;
#endif

/* 2.4 - 2.6 compatibility macros*/
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)

#define SK_ERR 		sk_err
#define SK_ERR_SOFT     sk_err_soft
#define SK_STATE_CHANGE sk_state_change
#define SK_PROT		sk_prot
#define SK_PROTOCOL	sk_protocol
#define SK_PROTINFO	sk_protinfo
#define SK_DESTRUCT	sk_destruct
#define SK_FAMILY	sk_family
#define SK_FLAGS	sk_flags
#define SK_DATA_READY   sk_data_ready
#define SK_RCVTIMEO     sk_rcvtimeo
#define SK_SNDTIMEO     sk_sndtimeo
#define SK_STATE        sk_state
#define SK_SLEEP        sk_sleep
#define SPORT(s)	(inet_sk(s))->num
#define SK_LINGER_FLAG(SK)      sock_flag((SK), SOCK_LINGER)
#define SK_LINGERTIME   sk_lingertime
#define SK_REUSE	sk_reuse
#define SK_SOCKET       sk_socket
#define SK_RCVBUF      sk_rcvbuf
#define SK_INET(s)      (inet_sk(s))

#else   /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) */

#define SK_ERR 		err
#define SK_ERR_SOFT     err_soft
#define SK_STATE_CHANGE state_change
#define SK_PROT		prot
#define SK_PROTOCOL	protocol
#define SK_PROTINFO	user_data
#define SK_DESTRUCT	destruct
#define SK_FAMILY	family
#define SK_FLAGS	dead
#define SK_DATA_READY   data_ready
#define SK_RCVTIMEO     rcvtimeo
#define SK_SNDTIMEO     sndtimeo
#define SK_STATE        state
#define SK_SLEEP        sleep
#define SPORT(s)	s->num
#define SK_LINGER_FLAG(SK)      ((SK)->linger)
#define SK_LINGERTIME   lingertime
#define SK_REUSE	reuse
#define SK_SOCKET       socket
#define SK_RCVBUF      rcvbuf
#define SK_INET(s)      s

#endif  /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) */

#define NOFC 0
#define FC1  1
#define FC2  2
#define FC3  3
#define FC4  4
#define FC5  5

/*
 * sk_alloc - varieties taken care here.
 */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,12)) || ((LINUX_VERSION_CODE == KERNEL_VERSION(2,6,11)) && (FEDORA == FC4))

#define	SK_ALLOC(FAMILY, PRIORITY, PROTO, ZERO_IT)		\
	sk_alloc((FAMILY), (PRIORITY), (PROTO), (ZERO_IT))

#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,9)

#define	SK_ALLOC(FAMILY, PRIORITY, PROTO, ZERO_IT)			\
	sk_alloc((FAMILY), (PRIORITY),					\
		 ((ZERO_IT) > 1) ? (PROTO)->slab_obj_size : (ZERO_IT),	\
		 (PROTO)->slab)

#else

#define	SK_ALLOC(FAMILY, PRIORITY, PROTO, ZERO_IT)	\
 	sk_alloc((FAMILY), (PRIORITY), sizeof(struct tcp_sock), nx_sk_cachep)

#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,14)
#define BIND_HASH(SK)   tcp_sk(SK)->bind_hash
#else
#define BIND_HASH(SK)   inet_csk(SK)->icsk_bind_hash
#endif

#if defined(CONFIG_X86_64)
#define PAGE_KERNEL_FLAG  PAGE_KERNEL_EXEC
#else 
#define PAGE_KERNEL_FLAG  PAGE_KERNEL
#endif


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,12)) || ((LINUX_VERSION_CODE == KERNEL_VERSION(2,6,11)) && (FEDORA == FC4))
#define	NX_KC_DST_MTU(DST)	dst_mtu((DST))
#else
#define	NX_KC_DST_MTU(DST)	dst_pmtu((DST))
#endif


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)

typedef struct work_struct	nx_kc_work_queue_t;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
#define	NX_KC_INIT_WORK(TASK, CB_FUNC, OBJ)	INIT_WORK((TASK), (CB_FUNC))
#else
#define	NX_KC_INIT_WORK(TASK, CB_FUNC, OBJ)	\
	INIT_WORK((TASK), (CB_FUNC), (OBJ))
#endif
#define	NX_KC_SCHEDULE_WORK(TASK)		schedule_work((TASK))
#define	NX_KC_SCHEDULE_DELAYED_WORK(TASK, DELAY)	\
	schedule_delayed_work((TASK), (DELAY))

#else

typedef struct tq_struct	nx_kc_work_queue_t;
#define	NX_KC_INIT_WORK(TASK, CB_FUNC, OBJ)	\
	INIT_TQUEUE((TASK), (CB_FUNC), (OBJ))
#define	NX_KC_SCHEDULE_WORK(TASK)	schedule_task((TASK))

#endif


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
#define PM_MESSAGE_T pm_message_t
#else
#define PM_MESSAGE_T u32
#endif

#ifndef module_param
#define module_param(v,t,p) MODULE_PARM(v, "i");
#endif


#ifdef __VMKERNEL_MODULE__
#define local_bh_enable()
#define local_bh_disable()
unsigned long virt_to_phys(void *address){return -1UL;}
#endif
#endif  /* __KERNEL_COMPATIBILITY_H__ */
