/*
 * Copyright (C) 2003 - 2007 NetXen, Inc.
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 * 
 * The full GNU General Public License is included in this distribution
 * in the file called LICENSE.
 * 
 * Contact Information:
 *    licensing@netxen.com
 * NetXen, Inc.
 * 3965 Freedom Circle, Fourth floor,
 * Santa Clara, CA 95054
 */
/*
 * The Linux API exported by the NetXen NIC driver to the LSA driver.
 */
#ifndef NX_NIC_LINUX_TNIC_API_H
#define NX_NIC_LINUX_TNIC_API_H

#include <linux/skbuff.h>
#include "nic_cmn.h"
#include "message.h"

typedef struct nx_tnic_adapter_s		nx_tnic_adapter_t;

/* 
 * Gets the tnic version number from the card. 
 */
typedef struct {
        __uint32_t	major;
        __uint32_t	minor;
        __uint32_t	sub;
} nic_version_t;

struct sk_buff;
typedef struct {
	int		size;		/* The size of this structure for NIC
					   to use if necessary. */
	nic_version_t	lsa_version;	/* For NIC to do checks on
					   Compatibility */
	int		(*nx_syn_handler)(struct net_device *, __uint16_t,
					  struct sk_buff *);
	void		(*nx_msg_handler)(struct net_device *, unm_msg_t *);
	int		(*nx_lsa_get_stats)(struct net_device *,
					    __uint8_t port,
					    struct net_device_stats *);
	int 		(*get_lsa_monitor_status)(nx_tnic_adapter_t *);
	int		(*lsa_setup_monitored_skb)(struct net_device *,
						   __uint32_t,
						   struct sk_buff *);
} nx_nic_lsa_registry_t;

/*
 * Checks if the specified device is a NetXen device.
 */
extern int __attribute__((weak)) nx_nic_is_netxen_device(struct net_device *netdev);

/*
 * Register Tnic driver with Nic driver.
 *
 * Return:
 *	0	- If registered successfully.
 *	1	- If already registered.
 *	-1	- If port or adapter is not set.
 */
extern int __attribute__((weak)) nx_nic_register_lsa(struct net_device *netdev,
			       nx_tnic_adapter_t *data,
			       nx_nic_lsa_registry_t *cb);
extern int __attribute__((weak)) nx_nic_get_adapter_revision_id (struct net_device *netdev);			       

/*
 * Unregisters a known LSA adapter.
 */
extern void __attribute__((weak)) nx_nic_unregister_lsa(struct net_device *netdev);

/*
 * Return the registered LSA adapter structure.
 *
 * Returns:
 *	The Toe adapter if already registered else returns NULL.
 */
extern nx_tnic_adapter_t * __attribute__((weak)) nx_nic_get_tnic_adapter(struct net_device *netdev);

/*
 * Gets the port number of the device on a netxen adaptor if it is a netxen
 * device.
 *
 * Parameters:
 *	dev	- The device for which the port is requested.
 *
 * Returns:
 *	-1	- If not netxen device.
 *	port number on the adapter if it is netxen device.
 */
extern int __attribute__((weak)) nx_nic_get_device_port(struct net_device *netdev);

/*
 * Gets the ring context used by the device to talk to the card.
 *
 * Parameters:
 *	dev	- The device for which the ring context is requested.
 *
 * Returns:
 *	-EINVAL	- If not netxen device.
 *	Ring context number if it is netxen device.
 */
extern int __attribute__((weak)) nx_nic_get_device_ring_ctx(struct net_device *netdev);

/*
 * Send a LSA message to the card.
 */
extern int __attribute__((weak)) nx_nic_send_tnic_msg(struct net_device *netdev,
				pegnet_cmd_desc_t *cmd_desc, int nr_elements);

/*
 * Gets the pointer to pci device if the device maps to a netxen device.
 */
extern struct pci_dev *__attribute__((weak)) nx_nic_get_pcidev(struct net_device *dev);

/*
 * Compare adapter ids for two NetXen devices.
 */
extern int __attribute__((weak)) nx_nic_cmp_adapter_id(struct net_device *dev1,
				 struct net_device *dev2);

/*
 * Gets the proc file directory where the procfs files are created.
 *
 * Parameters:
 *	None
 *
 * Returns:
 *	NULL - If the file system is not created.
 *	The directory that was created.
 */
struct proc_dir_entry *__attribute__((weak)) nx_nic_get_base_procfs_dir(void);

extern void __attribute__((weak)) nx_nic_get_tnic_version_number(struct net_device *netdev,
					   nic_version_t *tnic_version);
extern void __attribute__((weak)) nx_nic_get_nic_version_number(struct net_device *netdev,
					  nic_version_t *nic_version);

#endif	/* NX_NIC_LINUX_TNIC_API_H */
